#!/bin/sh

. "${_EXEC}"/pdiread.sh

edit_card(){
  local cardfile="$_DATA/vcard/$1" 
  local tempfile card

  . $_EXEC/session_lock.sh

  if ! tempfile="$(CHECK_SLOCK "$cardfile")"; then
    printf '[div .message %s]' "$(l10n "This card is not set up for editing within this session.")"
  else
    card="$(pdi_load "$tempfile")"
    cat <<-EOF
	[form .card #${cardfile##*/} action="/cards/update_card.sh" method="POST"
	  [input type="hidden" name="tid" value="$(transid ${tempfile})"]
	  [div .section .basic $(
	    edit_item "$card" N GENDER
	    [ "$(pdi_count "$card" NICKNAME)" -gt 0 ] \
	    && edit_item "$card" NICKNAME
	    edit_item "$card" BDAY
	    edit_item "$card" X-ZACK-JOINDATE
	    [ "$(pdi_count "$card" X-ZACK-LEAVEDATE)" -gt 0 ] \
	    && edit_item "$card" X-ZACK-LEAVEDATE
	    card_item "$card" SOUND PHOTO LOGO
	  )]
	  [div .section .phone   $(edit_item "$card" TEL)]
	  [div .section .message $(
	    edit_item "$card" EMAIL
	    [ $(pdi_count "$card" IMPP) -gt 0 ] && edit_item "$card" IMPP
	    [ $(pdi_count "$card" URL ) -gt 0 ] && edit_item "$card" URL
	  )]
	  [div .section .address $(edit_item "$card" ADR)]
	  [div .section .note    $(edit_item "$card" NOTE)]
	  [div .section .attendance
	    [h3 $(l10n course_attendance) ] $(
	    for course in "$_DATA"/ical/*.ics; do
	      printf '[label [input type="checkbox" name="attendance" value="%s" %s] %s]' \
	             "${course##*/}" \
	             "$(grep -qF "${course##*/}	${cardfile##*/}" "$_DATA/mappings/attendance" \
	                && printf 'checked="checked"'
	               )" \
	             "$(pdi_value "$(pdi_load "$course")" SUMMARY || l10n "(unnamed course)" |unescape |HTML)"
	    done)
	    [h3 $(l10n CATEGORIES) ] $(
	    grep -xE '[^ ]+' "$_DATA"/mappings/categories |while read -r cat; do
	      printf '[label [input type="checkbox" name="CATEGORIES" value="%s" %s] %s]' \
	             "$(HTML "$cat")" \
	             "$(seq 1 $(pdi_count "$card" CATEGORIES) |while read c; do
	               pdi_value "$card" CATEGORIES $c |grep -qxF "$cat" \
	               && printf 'checked="checked"' && break
	             done)" \
	             "$(HTML "$cat")"
	    done)
	  ]
	  [div .control
	    [div .item .delete label="$(l10n edit_delete)"
              [input type="checkbox" #delete]
              [label for="delete" $(l10n edit_delete)]
	      [button type="submit" name="action" value="delete" $(l10n edit_delete)]
            ]
            [div .item .newfield
              [select name="newfield"
	        [option value="" disabled="disabled" selected="selected" $(l10n edit_addfieldtext)]
	        $(for f in NICKNAME EMAIL TEL IMPP ADR URL NOTE; do
	          printf '[option value="%s" %s] ' "$f" "$(l10n "$f")"
	        done)
	      ][button type="submit" name="action" value="addfield" $(l10n edit_addfield)]
            ]
	    [button .item type="submit" name="action" value="update"   $(l10n edit_update)]
	    [button .item type="submit" name="action" value="cancel"   $(l10n edit_cancel)]
	  ]
	  [input type="hidden" name="UID" value="$(pdi_value "$card" UID |HTML)"]
	  [input type="hidden" name="card" value="${cardfile##*/}"]
	]
	EOF
  fi
}

print_card(){
  local cardfile="$1"
  local card="$(pdi_load "$cardfile")"
  cat <<-EOF
    [div .card #${cardfile##*/}
      [div .section .basic . $(
        card_item "$card" FN GENDER NICKNAME BDAY X-ZACK-JOINDATE X-ZACK-LEAVEDATE SOUND PHOTO LOGO
      )]
      [div .section .phone   . $(card_item "$card" TEL)]
      [div .section .message . $(card_item "$card" EMAIL IMPP URL)]
      [div .section .address . $(card_item "$card" ADR)]
      [div .section .note    . $(card_item "$card" NOTE)]
      [div .section .attendance [h3 $(l10n course_attendance) ] [ul
        $(grep -F "	${cardfile##*/}" "$_DATA/mappings/attendance" |while read each discard; do
          printf '[li [a .item .attendance href="/courses#%s" . %s]]' \
                 "$each" \
                 "$(pdi_value "$(pdi_load "$_DATA/ical/$each")" SUMMARY || l10n "(unnamed course)" |unescape |HTML)"
        done)]
        $(card_item "$card" CATEGORIES)
      ]
      [div .control
        [a .item href="/cards/edit_card.sh?card=${cardfile##*/}" $(l10n edit)]
        [a .item href="/cards/export_card.sh?card=${cardfile##*/}" $(l10n vcf_export)]
      ]
    ]
	EOF
}

print_cards(){
  local cardfile cachefile date size name ldate=0 lsize lname

  while read cardfile; do
    cachefile="${_DATA}/cache/${cardfile##*/}.cache"
    # if [ -s "$cachefile" -a "$cachefile" -nt "$cardfile" \
    #                      -a "$cachefile" -nt "${_EXEC}/cards" ]; then
    if [ -s "$cachefile" -a "$cachefile" -nt "$cardfile" ]; then
      cat "$cachefile"
    else
      print_card "$cardfile" |tee "$cachefile"
    fi
  done
}

filter_cards(){
  local filter f fex='x;p;'

  filter="$(printf %s "${filter}" \
            | sed -E 's;[]\/\(\)\\\$\?\.\+\*\;\[\{\}];\\\\&;g;
                      '"$upcase"
           )^"

  while [ "$filter" ]; do
    f="${filter%%^*}" filter="${filter#*^}"
    case $f in
      '') break
        ;;
      ANY:*) fex="/\n.*(\;[^:]*)?:[^\n]*(${f#*:})[^\n]*\r?\n/{${fex}}"
        ;;
      NAME:*) fex="/\n(N|FN|NICKNAME)(\;[^:]*)?:[^\n]*(${f#*:})[^\n]*\r?\n/{${fex}}"
        ;;
      STREET:*|ZIP:*) fex="/\nADR(\;[^:]*)?:[^\n]*(${f#*:})[^\n]*\r?\n/{${fex}}"
        ;;
      *) fex="/\n${f%%:*}(\;[^:]*)?:[^\n]*(${f#*:})[^\n]*\r?\n/{${fex}}"
        ;;
    esac
  done

  for cardfile in "${_DATA}"/vcard/*.vcf; do
    printf '%s\n' "$cardfile"
    cat "$cardfile"
  done \
  | sed -nE ':X; /\nEND:VCARD\r?$/!{ N; bX; }; h; s;\n.*$;;; x; s;^[^\n]+\n;;;
             '"$upcase""$fex"
}

order_cards() {
  local cardfile card

  while read cardfile; do
    card="$(pdi_load "$cardfile")"

    case $order in
      firstname)
        printf '%s	%s\n' "$(pdi_value "$card" FN)" "$cardfile"
        ;;
      lastname)
        printf '%s	%s\n' "$(pdi_value "$card" N || pdi_value "$card" FN)" "$cardfile"
        ;;
      bdate)
        printf '%s	%s\n' "$(pdi_value "$card" BDAY || printf 0000-00-00)" "$cardfile"
        ;;
    esac
  done \
  | sort \
  | sed -E 's;^.*\t;;g'
}

list_cards(){
  filter_cards \
  | order_cards \
  | grep -xvF "$edit" \
  | print_cards
}
